indexing
   description: "Fractal Types."
   status: "Free according to the GPL license (see COPYING)"
   date: "$Date$"
   revision: "$Revision$"
   author: "Mark Arrasmith <arrasmith@math.twsu.edu>"
   

class EFRACTAL_TYPE_FUNCTIONS

feature -- Fractal Type Functions

	mandel_julia(x,y,a,b: DOUBLE; iteration: INTEGER): INTEGER is
		-- z = z^2 + c
		require
			iteration >= 0
		local
			xn, yn, xo: DOUBLE
		do
			from
				xn := x
				yn := y
				Result := iteration
			until
				(xn*xn + yn*yn) > config.dump or Result = 0
			loop
				xo := xn
				xn := xn*xn - yn*yn + a
				yn := 2*xo*yn + b
				Result := Result - 1
			end
			config.set_color_x(xn)
			config.set_color_y(yn)
		end --mandel_julia

	forth_mandel_julia(x,y,a,b: DOUBLE; iteration: INTEGER): INTEGER is
		-- z = z^4 + c
		require
			iteration >= 0
		local
			xn, yn, xo: DOUBLE
		do
			from
				xn := x
				yn := y
				Result := iteration
			until
				(xn*xn + yn*yn) > config.dump or Result = 0
			loop
				xo := xn
				xn := xn*xn*xn*xn + yn*yn*yn*yn - 6*xn*xn*yn*yn + a
				yn := 4*xo*xo*xo*yn - 4*yn*yn*yn*xo + b
				Result := Result -1
			end
			config.set_color_x(xn)
			config.set_color_y(yn)
		end --forth_mandel_julia

	cube_mandel_julia(x,y,a,b: DOUBLE; iteration: INTEGER): INTEGER is
		-- z = z^3 + c
		require
			iteration >= 0
		local
			xn, yn, xo: DOUBLE
		do
			from
				xn := x
				yn := y
				Result := iteration
			until
				(xn*xn + yn*yn) > config.dump or Result = 0
			loop
				xo := xn
				xn := xn*(xn*xn - 3*yn*yn) + a
				yn := yn*(3*xo*xo - yn*yn) + b
				Result := Result - 1
			end
			config.set_color_x(xn)
			config.set_color_y(yn)
		end --cube_mandel_julia

	barnsley_1(x,y,a,b: DOUBLE; iteration: INTEGER): INTEGER is
		-- z = (z - 1)c if x_z >=0
		-- z = (z + 1)c if x_z < 0
		require
			iteration >= 0
		local
			xn, yn, xo: DOUBLE
		do
			from
				xn := x
				yn := y
				Result := iteration
			until
				(xn*xn + yn*yn) > config.dump or Result = 0
			loop
				xo := xn
				if xn < 0 then
					xn := (xn + 1)*a - yn*b
					yn := (xo + 1)*b + yn*a
				else
					xn := (xn - 1)*a - yn*b
					yn := (xo - 1)*b + yn*a
				end
				Result := Result - 1
			end
			config.set_color_x(xn)
			config.set_color_y(yn)
		end --barnsley_1

	barnsley_2(x,y,a,b: DOUBLE; iteration: INTEGER): INTEGER is
		-- z = (z - 1)c if x_z*y_c + x_c*y_z >=0
		-- z = (z + 1)c if x_z*y_c + x_c*y_z < 0
		require
			iteration >= 0
		local
			xn, yn, xo: DOUBLE
		do
			from
				xn := x
				yn := y
				Result := iteration
			until
				(xn*xn + yn*yn) > config.dump or Result = 0
			loop
				xo := xn
				if xn*b + yn*a < 0 then
					xn := (xn + 1)*a - yn*b
					yn := (xo + 1)*b + yn*a
				else
					xn := (xn - 1)*a - yn*b
					yn := (xo - 1)*b + yn*a
				end
				Result := Result - 1
			end
			config.set_color_x(xn)
			config.set_color_y(yn)
		end --barnsley_2

	barnsley_3(x,y,a,b: DOUBLE; iteration: INTEGER): INTEGER is
		-- z = (x*x - y*y - 1) + i(2*x*y) if x > 0
		-- z =  (x*x - y*y -1 + a*x) + i(2*x*y + b*x) if x <= 0
		require
			iteration >= 0
		local
			xn, yn, xo: DOUBLE
		do
			from
				xn := x
				yn := y
				Result := iteration
			until
				(xn*xn + yn*yn) > config.dump or Result = 0
			loop
				xo := xn
				if xn > 0 then
					xn := xn*xn - yn*yn - 1
					yn := 2*xo*yn
				else
					xn := xn*xn - yn*yn - 1 + a*xn
					yn := 2*xo*yn + b*xn
				end
				Result := Result - 1
			end
			config.set_color_x(xn)
			config.set_color_y(yn)
		end --barnsley_3

	oops(x,y,a,b: DOUBLE; iteration: INTEGER): INTEGER is
		-- my fist mistake at making a mandelbrot set
		require
			iteration >= 0
                local
                        xn, yn: DOUBLE
                do
                        from
                                xn := x
                                yn := y
                                Result := iteration
                        until
                                (xn*xn + yn*yn) > config.dump or Result = 0
                        loop
                                xn := xn*xn - yn*yn + a
                                yn := 2*xn*yn + b
                                Result := Result - 1
                        end
						config.set_color_x(xn)
						config.set_color_y(yn)
		end --oops

	manowar(x,y,a,b: DOUBLE; iteration: INTEGER): INTEGER is
		-- interesting fractal that looks like a manowar
		require
			iteration >= 0
                local
                        xn, yn, mx, my, xo, yo: DOUBLE
                do
                        from
                                xn := x
                                yn := y
                                mx := x
                                my := y
                                Result := iteration
                        until
                                (xn*xn + yn*yn) > config.dump or Result = 0
                        loop
                                xo := xn
								yo := yn
								xn := xo*xo - yo*yo + mx + a
								yn := 2*xo*yo + my + b
								mx := xo
								my := yo
                                Result := Result - 1
                        end 
						config.set_color_x(xn)
						config.set_color_y(yn)
		end --manowar

	magnet_1(x,y,a,b: DOUBLE; iteration: INTEGER): INTEGER is
		-- magnetic equation
		require
			iteration >= 0
                local
                        xn,yn,xt,yt,xb,yb,xr,yr : DOUBLE
                do
                        from
                                xn := x
                                yn := y
                                Result := iteration
                        until
                                (xn*xn + yn*yn) > config.dump or Result = 0
                        loop
								xt := xn*xn - yn*yn + a - 1
								yt := 2*xn*yn + b
								xb := 2*xn + a - 2
								yb := 2*yn + b
								if xb = 0 and yb = 0 then
								   Result := 0
								else
								xr := (xt*xb + yt*yb)/(xb*xb + yb*yb)
								yr := (xb*yt - yb*xt)/(xb*xb + yb*yb)
								xn := xr*xr - yr*yr
								yn := 2*xr*yr
								Result := Result - 1
								end
                        end
						config.set_color_x(xn)
						config.set_color_y(yn)
		end --magnet_1

	sine(x,y,a,b: DOUBLE; iteration: INTEGER): INTEGER is
		-- z = sin(z) + c
		require
			iteration >= 0
		local
			xn, yn, xo: DOUBLE
		do
			from
				xn := x
				yn := y
				Result := iteration
			until
				(xn*xn + yn*yn) > config.dump or Result = 0
			loop
				xo := xn
				xn := xn.sin*yn.cosh + a
				yn := xo.cos*yn.sinh + b
				Result := Result - 1
			end
			config.set_color_x(xn)
			config.set_color_y(yn)
		end --sine

	cosine(x,y,a,b: DOUBLE; iteration: INTEGER): INTEGER is
		-- z = cos(z) + c
		require
			iteration >= 0
		local
			xn, yn, xo: DOUBLE
		do
			from
				xn := x
				yn := y
				Result := iteration
			until
				(xn*xn + yn*yn) > config.dump or Result = 0
			loop
				xo := xn
				xn := xn.cos*yn.cosh + a
				yn := -(xo.sin)*yn.sinh + b
				Result := Result - 1
			end
			config.set_color_x(xn)
			config.set_color_y(yn)
		end --cosine
		
	c_cosine(x,y,a,b: DOUBLE; iteration: INTEGER): INTEGER is
		-- z = c*cos(z)
		require
			iteration >= 0
		local
			xn, yn, xo, yo: DOUBLE
		do
			from
				xn := x
				yn := y
				Result := iteration
			until
				(xn*xn + yn*yn) > config.dump or Result = 0
			loop
				xo := xn.cos*yn.cosh
				yo := -(xn.sin)*yn.sinh
				xn := a*xo - b*yo
				yn := a*yo + b*xo
				Result := Result - 1
			end
			config.set_color_x(xn)
			config.set_color_y(yn)
		end --c_cosine
		
	c_exp(x,y,a,b: DOUBLE; iteration: INTEGER): INTEGER is
		-- z = c*exp(z)
		require
			iteration >= 0
		local
			xn, yn, xo, yo: DOUBLE
		do
			from
				xn := x
				yn := y
				Result := iteration
			until
				(xn*xn + yn*yn) > config.dump or Result = 0
			loop
				xo := xn.exp*yn.cos
				yo := xn.exp*yn.sin
				xn := a*xo - b*yo
				yn := a*yo + b*xo
				Result := Result - 1
			end
			config.set_color_x(xn)
			config.set_color_y(yn)
		end --c_exp
		
	spider(x,y,a,b: DOUBLE; iteration: INTEGER): INTEGER is
		-- zn = z^2 + c
		-- cn = c/2 + zn
		require
			iteration >= 0
		local
			xn, yn, an, bn, xo: DOUBLE
		do
			from
				xn := x
				yn := y
				an := a
				bn := b
				Result := iteration
			until
				(xn*xn + yn*yn) > config.dump or Result = 0
			loop
				xo := xn
				xn := xn*xn - yn*yn + an
				yn := 2*xo*yn + bn
				an := an/2 + xn
				bn := bn/2 + yn
				Result := Result - 1
			end
			config.set_color_x(xn)
			config.set_color_y(yn)
		end --spider
		
	popcorn(x,y: DOUBLE; iteration: INTEGER): INTEGER is
		-- x = x - 0.05*sin(y) + tan(3y)
		-- y = y - 0.05 sin(x) + tan(3x)
		require
			iteration >= 0
		local
			xn, yn, xo: DOUBLE
		do
			from
				xn := x
				yn := y
				Result := iteration
			until
				(xn*xn + yn*yn) > config.dump or Result = 0
			loop
				xo := xn
				xn := xn - 0.05*yn.sin + (3*yn).tan
				yn := yn - 0.05*xo.sin + (3*xo).tan
				Result := Result - 1
			end
			config.set_color_x(xn)
			config.set_color_y(yn)
		end --c_cosine

	lyapunov(x,y: DOUBLE; iteration: INTEGER): INTEGER is
		-- two parameter cycling for a population model
		require
			iteration >= 0
		local
			p, summation, lyapunov_exponent: DOUBLE
			i: INTEGER
		do
			from
			  summation := 0.0
			  p := 0.5
			  i := iteration
			until
			  i = 0
			loop
			  if i > iteration/2 then
			    if config.lyapunov_pattern.item(((iteration - i)\\config.lyapunov_pattern.count)+1).same_as('1') then
				  p := x*p*(1 - p)
				else
				  p := y*p*(1 - p)
				end
			  else
			    if config.lyapunov_pattern.item(((iteration - i)\\config.lyapunov_pattern.count)+1).same_as('1') then
				  p := x*p*(1 - p)
				  summation := summation + (x - 2*x*p).abs.log
				else
				  p := y*p*(1 - p)
				  summation := summation + (y - 2*y*p).abs.log
				end
			  end
			  if summation.abs > 100000 or p.abs > 10000 then
			    i := 0
			  else
			  	i := i - 1
			  end
			end

			lyapunov_exponent := summation / iteration

			if lyapunov_exponent < 0 then
				Result := summation.rounded.abs
			else
				Result := 0
			end
		  config.set_color_x(x)
		  config.set_color_y(y)
		end --lyapunov

end --class EFRACTAL_TYPE_FUNCTIONS
