
indexing
   purpose: "modelates a selection object"
   
class RUBBER_BAND_SELECTION
   
inherit
   STARTUP
   GTK_CONSTANTS
   EXG_CONSTANTS
   
creation
   make
   
feature
   
   make (r: GDK_RECTANGLE) is
	 -- builds the selection object from the given rectangle
      local
	 p: GDK_POINT
      do
	 !ARRAY[GDK_POINT]! points.make (1, 0)
	 !! p.make (r.x, r.y + r.height - 1)
	 points.add_last (p)
	 !! p.make (r.x, r.y)
	 points.add_last (p)
	 !! p.make (r.x + r.width - 1, r.y)
	 points.add_last (p)
	 !! p.make (r.x + r.width - 1,
		    r.y + r.height - 1)
	 points.add_last (p)
      end
   
   draw (d: GDK_DRAWABLE_WRAPPER) is
	 -- draws the visual representation of the selection on the
	 -- given drawable
      local
	 gc: GDK_GC
	 i : INTEGER
	 cp, cp2, vp: GDK_POINT
	 vxd, vyd, magv, vx, vy, theta: DOUBLE
	 vtx, vty: INTEGER 
	 size: INTEGER
      do
	 !! gc.make (d)
	 gc.set_foreground (exg_white)
	 gc.set_function (gdk_xor)
	 gc.set_line_width (2)
	 from i := 1 until i > points.upper loop
	    cp := points.item (i)
	    if i = 1 then	     
	       size := Large_point_square_size
	    else
	       size := Point_square_size
	    end
	    d.draw_rectangle (gc, True,
			      cp.x - size, cp.y - size,
			      size * 2 + 1, size * 2 + 1)		      
	    i := i + 1
	 end
	 from i := 1 until i > points.upper-1 loop
	    cp := points.item (i)
	    cp2 := points.item (i + 1)
	    d.draw_line (gc, cp.x, cp.y, cp2.x, cp2.y)
	    i := i + 1
	 end
	 
	 cp := points.item (4)
	 cp2 := points.item (1)
	 d.draw_line (gc, cp.x, cp.y, cp2.x, cp2.y)
	 
	 cp := points.item (1)
	 cp2 := points.item (2)
	 vxd := (cp2.x - cp.x).to_double
	 vyd := (cp2.y - cp.y).to_double
	 magv := (vxd*vxd + vyd*vyd).sqrt
	 if magv = 0 then
	    vx := 0
		vy := 0
	 else
	    vx := vxd / magv
	    vy := vyd / magv
	 end
	 if vx = 0 then
	    theta := (Pi/2)*vyd.sign
	 else
	    theta := (vy/vx).atan
	 end
	 if vx < 0 then
	    theta := theta + Pi
	 end
	 !ARRAY[GDK_POINT]!arrow.make(1,0)
	 vtx := (cp.x + 10*(theta + 140 / Deg).cos + 0.3*vxd).rounded
	 vty := (cp.y + 10*(theta + 140 / Deg).sin + 0.3*vyd).rounded
	 !!vp.make(vtx,vty)
	 arrow.add_last(vp)
	 vtx := (cp.x + 10*(theta - 140 / Deg).cos + 0.3*vxd).rounded
	 vty := (cp.y + 10*(theta - 140 / Deg).sin + 0.3*vyd).rounded
	 !!vp.make(vtx,vty)
	 arrow.add_last(vp)
	 vtx := (cp.x + 0.3*vxd).rounded
	 vty := (cp.y + 0.3*vyd).rounded
	 !!vp.make(vtx,vty)
	 arrow.add_last(vp)
	 d.draw_polygon(gc, True, arrow)
	 gc.destroy
      end
   
   set_index (idx: INTEGER; pt: GDK_POINT) is
	 -- plays with the points to make `pt' have the index `idx'
      require
	 points.has (pt)
      local
	 actual_idx: INTEGER
      do
	 actual_idx := points.index_of (pt)
	 if actual_idx /= idx then
	    points.swap (actual_idx, idx)
	 end
      end
   
   rotate_left is
	 -- 2->1, 3->2, 4->3, 1->4
      local
	 i: INTEGER
	 aux: GDK_POINT
      do
	 aux := points.item (1)
	 from i := 1 until i > points.upper - 1 loop
	    points.put (points.item(i + 1), i)
	    i := i + 1
	 end
	 points.put (aux, points.upper)
      end
   
   rotate_right is
	 -- 1->2, 2->3, 3->4, 4->1
      local
	 i: INTEGER
	 aux: GDK_POINT
      do
	 aux := points.item (points.upper)
	 from i := points.upper until i < points.lower + 1 loop
	    points.put (points.item(i-1), i)
	    i := i - 1
	 end
	 points.put (aux, points.lower)
      end
   
feature -- queries & attributes
   
   matches_point (p: GDK_POINT): BOOLEAN is
	 -- checks if `p' is close to any of the `points' of the
	 -- selection'; if so, returns True and `matched_point' holds
	 -- the point that matched and `matched_point_index' its index
      local
	 i: INTEGER
	 size: INTEGER
      do
	 from i := 1 until i > points.upper loop
	    if i = 1 then	     
	       size := Large_point_square_size
	    else
	       size := Point_square_size
	    end
	    if (p.x - points.item (i).x).abs < size
	       and (p.y - points.item (i).y).abs < size then
	       Result := True
	       matched_point := points.item (i)
	       matched_point_index := i
	       i := points.upper
	    end
	    i := i + 1
	 end
      end
	 
   matched_point: GDK_POINT
   matched_point_index: INTEGER
   
   points: COLLECTION[GDK_POINT]
   arrow: COLLECTION[GDK_POINT]
   
end
	 
